@@ -272,7 +272,7 @@ traverseAlongPathOneStepBySyllables(const MatchedPinyinPath &path,
                 for (auto final : finals) {
                     updateNext(final, pos);
                 }
-            } else {
+            } else if (!path.flags_.test(PinyinDictFlag::FullMatch)) {
                 for (char test = PinyinEncoder::firstFinal;
                      test <= PinyinEncoder::lastFinal; test++) {
                     updateNext(std::make_pair(test, true), pos);
@@ -700,10 +700,16 @@ void PinyinDictionary::loadText(size_t idx, std::istream &in) {
                 prob = std::stof(tokens[2]);
             }
 
-            auto result = PinyinEncoder::encodeFullPinyin(pinyin);
-            result.push_back(pinyinHanziSep);
-            result.insert(result.end(), hanzi.begin(), hanzi.end());
-            trie.set(result.data(), result.size(), prob);
+            try {
+                auto result = PinyinEncoder::encodeFullPinyinWithFlags(
+                    pinyin, PinyinFuzzyFlag::VE_UE);
+                result.push_back(pinyinHanziSep);
+                result.insert(result.end(), hanzi.begin(), hanzi.end());
+                trie.set(result.data(), result.size(), prob);
+            } catch (const std::invalid_argument &e) {
+                LIBIME_ERROR()
+                    << "Failed to parse line: " << buf << ", skipping.";
+            }
         }
     }
     *mutableTrie(idx) = std::move(trie);
@@ -771,7 +777,8 @@ void PinyinDictionary::saveText(size_t idx, std::ostream &out) {
 
 void PinyinDictionary::addWord(size_t idx, std::string_view fullPinyin,
                                std::string_view hanzi, float cost) {
-    auto result = PinyinEncoder::encodeFullPinyin(fullPinyin);
+    auto result = PinyinEncoder::encodeFullPinyinWithFlags(
+        fullPinyin, PinyinFuzzyFlag::VE_UE);
     result.push_back(pinyinHanziSep);
     result.insert(result.end(), hanzi.begin(), hanzi.end());
     TrieDictionary::addWord(idx, std::string_view(result.data(), result.size()),
@@ -780,7 +787,8 @@ void PinyinDictionary::addWord(size_t idx, std::string_view fullPinyin,
 
 bool PinyinDictionary::removeWord(size_t idx, std::string_view fullPinyin,
                                   std::string_view hanzi) {
-    auto result = PinyinEncoder::encodeFullPinyin(fullPinyin);
+    auto result = PinyinEncoder::encodeFullPinyinWithFlags(
+        fullPinyin, PinyinFuzzyFlag::VE_UE);
     result.push_back(pinyinHanziSep);
     result.insert(result.end(), hanzi.begin(), hanzi.end());
     return TrieDictionary::removeWord(
